"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Just_1 = require("../../src/algebra/Just");
const compose2_1 = require("../../src/functions/compose2");
const composeUsingReverse_1 = require("../../src/functions/composeUsingReverse");
const curry_1 = require("../../src/functions/curry");
const identity_1 = require("../../src/functions/identity");
describe('Just is a Functor because: ', () => {
    it(' Just.map respects the Functor identity law for Just<T>', () => {
        const JustOf1 = Just_1.Just.of(1);
        expect(JustOf1.map(identity_1.identity).value).toBe(JustOf1.value);
    });
    it(' Just.map respects the Functor composition law for Just<T>', () => {
        const add1 = x => x + 1;
        const mult2 = x => x * 2;
        const add1ThenMult2 = composeUsingReverse_1.composeUsingReverse(mult2, add1);
        const JustOf1 = Just_1.Just.of(1);
        expect(JustOf1.map(add1ThenMult2).value).toEqual(JustOf1.map(add1).map(mult2).value);
    });
    it(' Just.map can map from Just<T> to Just<U> ', () => {
        class Dog {
            constructor(name) {
                this.name = name;
            }
            speaks() {
                return `${this.name} barks`;
            }
        }
        class Cat {
            constructor(name) {
                this.name = name;
            }
            speaks() {
                return `${this.name} meows`;
            }
        }
        function transform(dog) {
            return new Cat(dog.name);
        }
        const pointedFunctorOfDog = Just_1.Just.of(new Dog('Medor'));
        const pointedFunctorOfCat = pointedFunctorOfDog.map(transform);
        expect(pointedFunctorOfCat.map(identity_1.identity).value.speaks()).toEqual(`Medor meows`);
    });
});
describe('Just is a Monad because: ', () => {
    it(' Respects the Monad Left Identity Law', () => {
        const f = (x) => Just_1.Just.of(x.toUpperCase());
        const a = 'Hello';
        const lhs = Just_1.Just.of(a).chain(f);
        const rhs = f(a);
        expect(lhs).toEqual(rhs);
    });
    it(' Respects the Monad Right Identity Law', () => {
        const f = (x) => Just_1.Just.of(x);
        const ma = Just_1.Just.of('hello');
        const rhs = ma.chain(f);
        const lhs = ma;
        expect(lhs).toEqual(rhs);
    });
    it(' Respects the Monad Associativity Law', () => {
        const f = (x) => Just_1.Just.of(x.toUpperCase());
        const g = (x) => Just_1.Just.of(x.repeat(2));
        const ma = Just_1.Just.of('hello');
        const rhs = ma.chain(f).chain(g);
        const lhs = ma.chain(x => f(x).chain(g));
        expect(lhs).toEqual(rhs);
    });
});
describe('Just is an Applicative because: ', () => {
    it(' Respects the Identity Law', () => {
        const applicativeOfId = Just_1.Just.of(identity_1.identity);
        const functorOf4 = Just_1.Just.of(4);
        expect(applicativeOfId.ap(functorOf4)).toEqual(functorOf4);
    });
    it(' Respects the Homomorphism Law', () => {
        const f = x => x + 2;
        const applicativeOfF = Just_1.Just.of(f);
        const applicativeOf1 = Just_1.Just.of(1);
        expect(applicativeOfF.ap(applicativeOf1)).toEqual(Just_1.Just.of(f(1)));
    });
    it(' Respects the Interchange Law', () => {
        const f = x => x + 2;
        const applicativeOfF = Just_1.Just.of(f);
        const applicativeOf1 = Just_1.Just.of(1);
        expect(applicativeOfF.ap(Just_1.Just.of(1))).toEqual(Just_1.Just.of(f).ap(applicativeOf1));
    });
    it(' Respects the Composition Law', () => {
        const compose = curry_1.curry(compose2_1.compose2);
        const u = Just_1.Just.of((x) => x.toUpperCase());
        const v = Just_1.Just.of((x) => x.concat('& beyond'));
        const w = Just_1.Just.of('blood bath');
        expect(Just_1.Just.of(compose)
            .ap(u)
            .ap(v)
            .ap(w)).toEqual(u.ap(v.ap(w)));
    });
});
//# sourceMappingURL=Just.spec.js.map